@@ -3,10 +3,10 @@
 # Date created: 2018-09-10
 
 
-#' Identify comparisons in loops that fulfill criteria for node-splitting
+#' Identify comparisons in loops that fulfil criteria for node-splitting
 #'
 #' Identify comparisons informed by both direct and indirect evidence from
-#' independent sources, which therefore fulfill the criteria for testing for
+#' independent sources, which therefore fulfil the criteria for testing for
 #' inconsistency via node-splitting. Follows the method of van Valkenhoef \insertCite{vanvalkenhoef2016;textual}{MBNMAtime}.
 #'
 #' @param data A data frame containing variables `studyID` and `treatment` (as
@@ -249,11 +249,11 @@ mb.comparisons <- function(data)
 }
 
 
-#' Identify comparisons in time-course MBNMA datasets that fulfill criteria for node-splitting
+#' Identify comparisons in time-course MBNMA datasets that fulfil criteria for node-splitting
 #'
 #' Identify comparisons informed by both direct and indirect evidence from independent sources in MBNMA
 #' datasets with repeated measurements in each study. These comparisons are therefore those which
-#' fulfill the criteria for testing for inconsistency via node-splitting, following the method of van
+#' fulfil the criteria for testing for inconsistency via node-splitting, following the method of van
 #' Valkenhoef \insertCite{vanvalkenhoef2016;textual}{MBNMAtime}.
 #'
 #' @inheritParams mb.run
@@ -304,7 +304,6 @@ mb.nodesplit.comparisons <- function(network)
 #' @param nodesplit.parameters A character vector of named time-course parameters on which to
 #' node-split (e.g. c("beta.1", "beta.2")). Can use "all" to split on all time-course parameters.
 #' @param ... Arguments to be sent to `mb.run()`
-#' @inheritParams mb.emax.hill
 #' @inheritParams mb.run
 #'
 #' @inherit mb.run details
@@ -328,22 +327,21 @@ mb.nodesplit.comparisons <- function(network)
 #' @examples
 #' \donttest{
 #' # Create mb.network object
-#' network <- mb.network(osteopain)
+#' painnet <- mb.network(osteopain)
 #'
 #' # Identify comparisons informed by direct and indirect evidence
-#' splits <- mb.nodesplit.comparisons(network)
+#' splits <- mb.nodesplit.comparisons(painnet)
 #'
-#' # Fit an exponential time-course MBNMA
-#' result <- mb.nodesplit(network, comparisons=splits, nodesplit.parameters="all",
-#'   fun="exponential",
-#'   beta.1=list(pool="rel", method="common"))
+#' # Fit a log-linear time-course MBNMA (takes a while to run)
+#' result <- mb.nodesplit(painnet, comparisons=splits, nodesplit.parameters="all",
+#'   fun=tloglin(pool.rate="rel", method.rate="common"),
+#'   rho="dunif(0,1)", covar="varadj"
+#'   )
 #'
 #' # Fit an emax time-course MBNMA with a node-split on emax parameters only
-#' result <- mb.nodesplit(network, comparisons=splits, nodesplit.parameters="beta.1",
-#'   fun="emax",
-#'   beta.1=list(pool="rel", method="random"),
-#'   beta.2=list(pool="rel", method="common")
-#'   )
+#' result <- mb.nodesplit(painnet, comparisons=splits, nodesplit.parameters="emax",
+#'   fun=temax(pool.emax="rel", method.emax="common",
+#'     pool.et50="rel", method.et50="common"))
 #'
 #' # Inspect results
 #' print(result)
@@ -354,8 +352,7 @@ mb.nodesplit.comparisons <- function(network)
 #' }
 #' @export
 mb.nodesplit <- function(network, comparisons=mb.nodesplit.comparisons(network),
-                            nodesplit.parameters="all", fun="linear", user.fun=NULL,
-                            beta.1=list(pool="rel", method="common"), beta.2=NULL, beta.3=NULL, beta.4=NULL,
+                            nodesplit.parameters="all", fun=tpoly(degree = 1),
                             ...
 )
 {
@@ -373,65 +370,38 @@ mb.nodesplit <- function(network, comparisons=mb.nodesplit.comparisons(network),
   checkmate::reportAssertions(argcheck)
 
   # Check betas are specified correctly and prepare format for subsequent functions
-  for (i in 1:4) {
-    betaname <- paste0("beta.", i)
-    if (!is.null(get(betaname))) {
-      assign(paste0(betaname, ".str"), compound.beta(get(betaname)))
-    } else if (is.null(get(betaname))) {
-      assign(paste0(betaname, ".str"), NULL)
-    }
-  }
+  # for (i in 1:4) {
+  #   betaname <- paste0("beta.", i)
+  #   if (!is.null(get(betaname))) {
+  #     assign(paste0(betaname, ".str"), compound.beta(get(betaname)))
+  #   } else if (is.null(get(betaname))) {
+  #     assign(paste0(betaname, ".str"), NULL)
+  #   }
+  # }
 
   # Get treatment labels
   trt.labs <- network$treatments
 
-
-  # Checks - NEEDS TO BE CHANGED TO REFLECT MODEL CODE...COULD HAPPEN AFTER mb.write and then
-  #used to look inside the written model code to check numbers of parameters etc.
-  # "d.2" is not a parameter in current model with single prior - it is "beta.2"
-  #if (is.null(parameters.to.save)) {
-  #  if (fun=="linear" | fun=="exponential") {
-  #    parameters.to.save=c("d.1")
-  #  } else if (fun=="emax") {
-  #    parameters.to.save=c("d.1", "d.2")
-  #  }
-  #}
-
   # Check nodesplit.parameters has true values
   if (is.null(nodesplit.parameters)) {
     stop("No parameters have been specified on which to node-split")
   }
 
-  if (!all(nodesplit.parameters %in% c("all", "beta.1", "beta.2", "beta.3", "beta.4"))) {
-    stop("Parameter specified for nodesplit.parameters does not exist in model. They must be of the type 'beta.1', 'beta.2', etc.")
+  if (!all(nodesplit.parameters %in% c("all", fun$params[which(fun$apool=="rel")]))) {
+    stop("Parameter specified for nodesplit.parameters must be a parameter modelled using relative effects specified\nwithin the model")
   }
 
-  # Check that nodesplit.parameters and class effects are not on same parameter
-  #for (i in seq_along(names(class.effect))) {
-  #  if (names(class.effect)[i] %in% nodesplit.parameters) {
-  #    stop(paste0("Node splitting cannot be applied to parameters with class effects. ", names(class.effect)[i], " has been given a class effect"))
-  #  }
-  #}
 
   if (nodesplit.parameters=="all") {
     UME <- TRUE
-    nodesplit.parameters <- vector()
-    treat.params <- c("beta.1", "beta.2", "beta.3", "beta.4")
-
-    for (i in seq_along(treat.params)) {
-      if (!is.null(get(treat.params[i]))) {
-        if (get(treat.params[i])$pool == "rel") {
-          nodesplit.parameters <- append(nodesplit.parameters, treat.params[i])
-        }
-      }
-    }
-
+    nodesplit.parameters <- fun$params[which(fun$apool=="rel")]
   } else {
     UME <- nodesplit.parameters
   }
 
-
-  # Separate ... arguments for mb.write and mb.run
+  if (length(nodesplit.parameters)==0) {
+    stop("Parameter specified for nodesplit.parameters must be a parameter modelled using relative effects specified\nwithin the model")
+  }
 
 
   ########### CHECKS OF DATASET FOR VALIDITY OF NODE-SPLITTING (possibly use Val Valkenhoef automation) ############
@@ -441,38 +411,14 @@ mb.nodesplit <- function(network, comparisons=mb.nodesplit.comparisons(network),
   comparisons[["t1"]] <- as.numeric(comparisons[["t1"]])
   comparisons[["t2"]] <- as.numeric(comparisons[["t2"]])
   for (i in seq_along(comparisons[["t1"]])) {
-    comparisons[i,1:2] <- sort(comparisons[i,1:2])
+    comparisons[i,1:2] <- sort(as.matrix(comparisons[,1:2])[i,])
   }
 
 
-  ######## Write JAGS scripts ########
-
-  #These can be removed later as now incorporated into mb.run
-  # Only needs to capture key time-course parameters in model for gen.paramaters.to.save
-  model.ind <- mb.write(fun=fun, user.fun=user.fun, beta.1=beta.1.str, beta.2=beta.2.str,
-                           beta.3=beta.3.str, beta.4=beta.4.str, UME=FALSE
-                           )
-
-  #model.dir <- mb.write(fun=fun, user.fun=user.fun, alpha=alpha, beta.1=beta.1, beta.2=beta.2,
-  #                         beta.3=beta.3, beta.4=beta.4,
-  #                         positive.scale=positive.scale, intercept=intercept, rho=rho, covar=covar,
-  #                         class.effect=class.effect, UME=UME)
-
-  # parameters.to.save <- gen.parameters.to.save(parameters.to.save=parameters.to.save,
-  #                                              model.params = c(1,2,3,4),
-  #                                              model = model.ind
-  # )
-
-  parameters.to.save <-
-    gen.parameters.to.save(model.params=c(1,2,3,4), model=model.ind)
-
-
   ############# Run NMA model #############
 
-  result.nma <- mb.run(network, parameters.to.save=parameters.to.save, fun=fun,
-                       beta.1=beta.1, beta.2=beta.2, beta.3=beta.3, beta.4=beta.4,
-                       ...
-                       )
+  result.nma <- do.call(mb.run, args=list(network=network, fun=fun,
+                                         ...))
 
 
   ######### Loop over all node splits in network ########
@@ -492,19 +438,20 @@ mb.nodesplit <- function(network, comparisons=mb.nodesplit.comparisons(network),
     message("Running NMA model")
 
     nma.dif <- list()
-    nodesplit.parameters <- sort(nodesplit.parameters)
-
-    for (param in 1:4) {
-      if (paste0("beta.", param) %in% nodesplit.parameters) {
-
-        node1 <- paste0("d.", param, "[", comp[1], "]")
-        node2 <- paste0("d.", param, "[", comp[2], "]")
-
-        nma1 <- result.nma$BUGSoutput$sims.matrix[,colnames(result.nma$BUGSoutput$sims.matrix)==node1]
-        nma2 <- result.nma$BUGSoutput$sims.matrix[,colnames(result.nma$BUGSoutput$sims.matrix)==node2]
-
-        nma.dif[[paste0("beta.", param)]] <- nma2 - nma1
+    for (param in seq_along(nodesplit.parameters)) {
+      if (grepl("beta", nodesplit.parameters[param])) {
+        index <- which(nodesplit.parameters[param] %in% fun$params)
+        node1 <- paste0("d.", index, "[", comp[1], "]")
+        node2 <- paste0("d.", index, "[", comp[2], "]")
+      } else{
+        node1 <- paste0(nodesplit.parameters[param], "[", comp[1], "]")
+        node2 <- paste0(nodesplit.parameters[param], "[", comp[2], "]")
       }
+
+      nma1 <- result.nma$BUGSoutput$sims.matrix[,colnames(result.nma$BUGSoutput$sims.matrix)==node1]
+      nma2 <- result.nma$BUGSoutput$sims.matrix[,colnames(result.nma$BUGSoutput$sims.matrix)==node2]
+
+      nma.dif[[nodesplit.parameters[param]]] <- nma2 - nma1
     }
 
 
@@ -512,18 +459,6 @@ mb.nodesplit <- function(network, comparisons=mb.nodesplit.comparisons(network),
     ######### For direct model  ###########
     #######################################
 
-    # dir.dif <- list()
-    # nodesplit.parameters <- sort(nodesplit.parameters)
-    #
-    # for (param in 1:4) {
-    #   if (paste0("beta.", param) %in% nodesplit.parameters) {
-    #     node <- paste0("d.", param, "[", comp[1], ",", comp[2], "]")
-    #     #dir.dif[[length(dir.dif)+1]] <-
-    #     dir.dif[[paste0("beta.", param)]] <-
-    #       result.dir$BUGSoutput$sims.matrix[,colnames(result.dir$BUGSoutput$sims.matrix)==node]
-    #   }
-    # }
-
     # Change network reference treatment to estimate direct effects
     if (!exists("results.dir") |
         network.temp$treatment[1] != network$treatment[comp[1]]) {
@@ -535,25 +470,23 @@ mb.nodesplit <- function(network, comparisons=mb.nodesplit.comparisons(network),
       }
 
       # Run UME model to estimate direct effects
-      result.dir <- mb.run(network.temp, fun=fun, parameters.to.save=parameters.to.save,
-                              beta.1=beta.1, beta.2=beta.2, beta.3=beta.3, beta.4=beta.4,
-                              UME=UME#,
-                              #...
-      )
+      result.dir <- do.call(mb.run, args=list(network=network.temp, fun=fun, UME=UME,
+                                              ...))
     }
 
     # Store required model parameter values
     comp.temp <- which(network$treatments[comp[2]] == network.temp$treatments)
     dir.dif <- list()
-    nodesplit.parameters <- sort(nodesplit.parameters)
-
-    for (param in 1:4) {
-      if (paste0("beta.", param) %in% nodesplit.parameters) {
-        node <- paste0("d.", param, "[1,", comp.temp, "]")
-        #dir.dif[[length(dir.dif)+1]] <-
-        dir.dif[[paste0("beta.", param)]] <-
-          result.dir$BUGSoutput$sims.matrix[,colnames(result.dir$BUGSoutput$sims.matrix)==node]
+    for (param in seq_along(nodesplit.parameters)) {
+      if (grepl("beta", nodesplit.parameters[param])) {
+        index <- which(nodesplit.parameters[param] %in% fun$params)
+        node <- paste0("d.", index, "[1,", comp.temp, "]")
+      } else{
+        node <- paste0(nodesplit.parameters[param], "[1,", comp.temp, "]")
       }
+
+      dir.dif[[nodesplit.parameters[param]]] <-
+        result.dir$BUGSoutput$sims.matrix[,colnames(result.dir$BUGSoutput$sims.matrix)==node]
     }
 
     message("Direct complete")
@@ -566,39 +499,36 @@ mb.nodesplit <- function(network, comparisons=mb.nodesplit.comparisons(network),
     ####### For MBNMA indirect model ########
     #########################################
 
-    data <- network[["data.ab"]]
+    df <- network[["data.ab"]]
 
     # Remove comparisons to split on
-    data <- drop.comp(df=data, comp=comp)
+    df <- drop.comp(df=df, comp=comp)
 
-    if (!(1 %in% data$treatment)) {
+    if (!(1 %in% df$treatment)) {
       string <- paste("Reference treatment removed for node-split. Treatments have been reordered with the next lowest coded treatment as the reference:\ntreatment ",
-                     min(data$treatment, na.rm=TRUE))
+                     min(df$treatment, na.rm=TRUE))
       warning(string)
     }
 
-    data <- mb.network(data)
+    ind.net <- mb.network(df)
 
-    result.ind <- mb.run(data, parameters.to.save=parameters.to.save, fun=fun,
-                            beta.1=beta.1, beta.2=beta.2, beta.3=beta.3, beta.4=beta.4#,
-                            #...
-    )
+    result.ind <- do.call(mb.run, args=list(network=ind.net, fun=fun, ...))
 
     ind.dif <- list()
-    nodesplit.parameters <- sort(nodesplit.parameters)
-
-    for (param in 1:4) {
-      if (paste0("beta.", param) %in% nodesplit.parameters) {
-
-        node1 <- paste0("d.", param, "[", comp[1], "]")
-        node2 <- paste0("d.", param, "[", comp[2], "]")
-
-        ind1 <- result.ind$BUGSoutput$sims.matrix[,colnames(result.ind$BUGSoutput$sims.matrix)==node1]
-        ind2 <- result.ind$BUGSoutput$sims.matrix[,colnames(result.ind$BUGSoutput$sims.matrix)==node2]
-
-        #ind.dif[[length(ind.dif)+1]] <- ind2 - ind1
-        ind.dif[[paste0("beta.", param)]] <- ind2 - ind1
+    for (param in seq_along(nodesplit.parameters)) {
+      if (grepl("beta", nodesplit.parameters[param])) {
+        index <- which(nodesplit.parameters[param] %in% fun$params)
+        node1 <- paste0("d.", index, "[", comp[1], "]")
+        node2 <- paste0("d.", index, "[", comp[2], "]")
+      } else{
+        node1 <- paste0(nodesplit.parameters[param], "[", comp[1], "]")
+        node2 <- paste0(nodesplit.parameters[param], "[", comp[2], "]")
       }
+
+      ind1 <- result.ind$BUGSoutput$sims.matrix[,colnames(result.ind$BUGSoutput$sims.matrix)==node1]
+      ind2 <- result.ind$BUGSoutput$sims.matrix[,colnames(result.ind$BUGSoutput$sims.matrix)==node2]
+
+      ind.dif[[nodesplit.parameters[param]]] <- ind2 - ind1
     }
 
 
@@ -617,7 +547,19 @@ mb.nodesplit <- function(network, comparisons=mb.nodesplit.comparisons(network),
 
       overlap.mat <- list("direct"=dir.dif[[i]], "indirect"=ind.dif[[i]])
       overlap <- overlapping::overlap(overlap.mat, plot=FALSE)
-      p.values <- overlap$OV
+      #p.values <- overlap$OV
+      diff <- sum((dir.dif[[i]]-ind.dif[[i]])>0) / length(dir.dif[[i]])
+      p.values <- min(diff, 1-diff)
+
+      plot.df <- data.frame(source=c(rep("NMA", length(nma.dif[[i]])),
+                                     rep("Direct", length(dir.dif[[i]])),
+                                     rep("Indirect", length(ind.dif[[i]]))
+                                     ),
+                            value=c(nma.dif[[i]],
+                                    dir.dif[[i]],
+                                    ind.dif[[i]]
+                                    )) %>%
+        dplyr::mutate(Source=factor(source))
 
 
       # Quantiles
@@ -628,70 +570,24 @@ mb.nodesplit <- function(network, comparisons=mb.nodesplit.comparisons(network),
       quantiles <- list("difference" = quantile_dif, "direct"=quantile_dir, "indirect"=quantile_ind, "nma"=quantile_nma)
 
 
-      # GGplots
-
-      source <- c("NMA", "Direct", "Indirect")
-      l95 <- c(quantile_nma[1], quantile_dir[1], quantile_ind[1])
-      med <- c(quantile_nma[2], quantile_dir[2], quantile_ind[2])
-      u95 <- c(quantile_nma[3], quantile_dir[3], quantile_ind[3])
-      plotdata <- data.frame(source, l95, med, u95)
-
-      #title <- paste0(names(ind.dif)[i], ": Treatment ", comp[2], " vs Treatment ", comp[1])
-      title <- paste0(names(ind.dif)[i], ": ", trt.labs[comp[2]], " vs ", trt.labs[comp[1]])
-
-      gg <-
-        ggplot2::ggplot(data=plotdata, ggplot2::aes(x=source, y=med, ymin=l95, ymax=u95)) +
-        ggplot2::geom_pointrange() +
-        #ggplot2::geom_hline(yintercept=0, lty=2) +
-        ggplot2::coord_flip() +  # flip coordinates (puts labels on y axis)
-        ggplot2::xlab("") + ggplot2::ylab("Treatment effect (95% CrI)") + ggplot2::ggtitle(title) +
-        ggplot2::theme(axis.text = ggplot2::element_text(size=15),
-                       axis.title = ggplot2::element_text(size=18),
-                       title=ggplot2::element_text(size=18)) +
-        ggplot2::theme(plot.margin=ggplot2::unit(c(1,1,1,1),"cm")) +
-        ggplot2::theme_bw()
-
-
-      # Density plots (with shaded area of overlap)
-      #MBNMA <- Sowers2005_nma[1:length(Direct)]
-      molten <- data.frame(ind.dif[[i]], dir.dif[[i]])
-      molten <- reshape2::melt(molten, measure.vars=names(molten))
-      names(molten) <- c("Estimate", "value")
-      linetypes <- c("solid", "dash")
-      levels(molten$Estimate) <- c("Indirect", "Direct")
-
-      dens <- ggplot2::ggplot(molten, ggplot2::aes(x=molten$value, linetype=molten$Estimate, fill=molten$Estimate)) +
-        ggplot2::geom_density(alpha=0.2) +
-        ggplot2::xlab(title) +
-        ggplot2::ylab("Posterior density") +
-        #scale_linetype_manual(values=factor(molten$Estimate)) +
-        #scale_fill_manual(name="Evidence") +
-        ggplot2::theme(strip.text.x = ggplot2::element_text(size=12)) +
-        ggplot2::theme(axis.text = ggplot2::element_text(size=12),
-                       axis.title = ggplot2::element_text(size=14)) +
-        ggplot2::theme_bw()
-
-
       # Add plots for overlap and forest in return
       nodesplit <- list("comparison"= c(trt.labs[comp[2]], trt.labs[comp[1]]),
-                        "parameter"=paste("d.", i,
+                        "parameter"=paste(names(ind.dif)[i],
                                           paste("[",comp[1], ",", comp[2], "]", sep=""),
                                           sep=""),
                         "overlap matrix"=overlap.mat,
                         "p.values"=p.values, "quantiles"=quantiles,
-                        "forest.plot"=gg, "density.plot"=dens,
-                        "direct"=result.dir, "indirect"=result.ind)
+                        #"forest.plot"=gg, "density.plot"=dens,
+                        "mcmc"=plot.df)
 
-      #nodesplit.comparison[[length(nodesplit.comparison)+1]] <- nodesplit
       nodesplit.comparison[[names(ind.dif)[i]]] <- nodesplit
 
     }
 
-    nodesplit.results[[paste("d",comp[1],comp[2], sep=".")]] <- nodesplit.comparison
-
+    nodesplit.results[[paste(trt.labs[comp[2]], trt.labs[comp[1]], sep=" vs ")]] <- nodesplit.comparison
   }
 
-  class(nodesplit.results) <- "mb.nodesplit"
+  class(nodesplit.results) <- "nodesplit"
 
   return(nodesplit.results)
 }
@@ -730,57 +626,6 @@ drop.comp <- function(df, comp) {
 
 
 
-# FUNCTION IS DEPRACATED!!!
-# check.path <- function(data, dropdata,
-#                        comparisons=c(comparisons[i,1], comparisons[i,2]),
-#                        path, graph) {
-#   # Identify if there is still an indirect pathway not involving direct evidence studies
-#
-#   # Comparisons is a numeric vector of length 2 indicating the treatment comparison on which to be split
-#
-#   # Then check for new paths...
-#
-#   path.fail <- TRUE
-#
-#   while(as.logical(is.finite(igraph::shortest.paths(
-#     graph, comparisons[1], comparisons[2]))) == TRUE) {
-#
-#     # Delete current path from graph to check for next shortest path
-#     for (edge in 1:length(path)-1) {
-#       del.index <- which(apply(igraph::as_edgelist(graph), 1,
-#                                function(x) identical(x, as.character(path[edge:(edge+1)]))))
-#       graph <- igraph::as.undirected(igraph::delete_edges(graph, del.index))
-#     }
-#
-#     # Check if the indirect path is not included in any study that has been dropped (for node-splitting)
-#     # And if so break from the function with a pass
-#     if (any(apply(dropdata,1, function(x) all(path %in% x[["design"]]))) == FALSE) {
-#       path.fail <- FALSE
-#       break()
-#       #} else {
-#       #  path <- as.numeric(igraph::shortest_paths(igraph::as.undirected(graph),
-#       #                                            comparisons[1], comparisons[2],
-#       #                                            weights=NA
-#       #  )[["vpath"]][[1]])
-#       #}
-#
-#       # ADD THIS SECTION IF GETTING WARNINGS FROM IGRAPH
-#     } else if (as.logical(is.finite(igraph::shortest.paths(
-#       graph, comparisons[1], comparisons[2], weights=NULL))) == TRUE){
-#
-#       path <- as.numeric(igraph::shortest_paths(igraph::as.undirected(graph),
-#                                                 comparisons[1], comparisons[2],
-#                                                 weights=NA
-#       )[["vpath"]][[1]])
-#     } else {break()}
-#
-#   }
-#
-#   return(list("path.fail"=path.fail, "path"=path))
-# }
-
-
-
 
 # FUNCTION IS DEPRACATED!!!
 check.design <- function(data, path) {
